"use strict";

var _excluded = ["className", "style", "label", "help", "hasFeedback", "name", "required", "noStyle", "disabled", "rules", "children", "messageVariables", "trigger", "validateTrigger"];

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FormItem = void 0;

var react_1 = __importDefault(require("react"));

var classnames_1 = __importDefault(require("classnames"));

var rc_field_form_1 = require("rc-field-form");

var FieldContext_1 = __importDefault(require("rc-field-form/lib/FieldContext"));

var context_1 = require("./context");

var utils_1 = require("./utils");

var classPrefix = "am-form-item";
var MemoInput = react_1["default"].memo(function (_ref) {
  var children = _ref.children;
  return children;
}, function (prev, next) {
  return prev.value === next.value && prev.update === next.update;
});

var FormItemLayout = function FormItemLayout(props) {
  var _classnames_1$default;

  var className = props.className,
      style = props.style,
      label = props.label,
      help = props.help,
      required = props.required,
      disabled = props.disabled,
      meta = props.meta,
      children = props.children,
      htmlFor = props.htmlFor;
  var hasFeedback = props.hasFeedback || react_1["default"].useContext(context_1.FormContext).hasFeedback;
  var formItemClass = classnames_1["default"](classPrefix, className);
  var formItemLabelClass = classnames_1["default"](classPrefix + "-label", (_classnames_1$default = {}, _classnames_1$default[classPrefix + "-label-disable"] = disabled, _classnames_1$default));
  var feedback = hasFeedback && meta && meta.errors.length > 0 ? meta.errors[0] : null;
  return react_1["default"].createElement("div", {
    className: formItemClass,
    style: style
  }, react_1["default"].createElement("label", {
    className: formItemLabelClass,
    htmlFor: htmlFor
  }, required && react_1["default"].createElement("span", {
    className: classPrefix + "-label-required"
  }, "*"), label, help && react_1["default"].createElement("span", {
    className: classPrefix + "-label-help"
  }, help)), react_1["default"].createElement("div", {
    className: classPrefix + "-body"
  }, children), feedback && react_1["default"].createElement("div", {
    className: classPrefix + "-footer"
  }, feedback));
};

exports.FormItem = function (props) {
  var className = props.className,
      style = props.style,
      displayLabel = props.label,
      help = props.help,
      hasFeedback = props.hasFeedback,
      name = props.name,
      required = props.required,
      noStyle = props.noStyle,
      disabled = props.disabled,
      rules = props.rules,
      children = props.children,
      messageVariables = props.messageVariables,
      _props$trigger = props.trigger,
      trigger = _props$trigger === void 0 ? 'onChange' : _props$trigger,
      validateTrigger = props.validateTrigger,
      fieldProps = _objectWithoutPropertiesLoose(props, _excluded);

  var _react_1$default$useC = react_1["default"].useContext(FieldContext_1["default"]),
      contextValidateTrigger = _react_1$default$useC.validateTrigger;

  var mergedValidateTrigger = validateTrigger !== undefined ? validateTrigger : contextValidateTrigger;
  var updateRef = react_1["default"].useRef(0);
  updateRef.current += 1; // 如果 label 未设置，那么尝试使用 name 作为 label

  var label = displayLabel ? displayLabel : typeof name === 'string' ? name : undefined;

  function renderLayout(baseChildren, fieldId, meta, isRequired) {
    if (noStyle) {
      return baseChildren;
    }

    return react_1["default"].createElement(FormItemLayout, {
      className: className,
      style: style,
      label: label,
      help: help,
      required: isRequired,
      disabled: disabled,
      hasFeedback: hasFeedback,
      htmlFor: fieldId,
      meta: meta
    }, baseChildren);
  }

  var isRenderProps = typeof children === 'function';

  if (!name && !isRenderProps && !props.dependencies) {
    return renderLayout(children);
  }

  var variables = {};

  if (typeof label === 'string') {
    variables.label = label;
  }

  if (messageVariables) {
    variables = _extends({}, variables, messageVariables);
  }

  return react_1["default"].createElement(rc_field_form_1.Field, Object.assign({}, fieldProps, {
    name: name,
    rules: rules,
    trigger: trigger,
    validateTrigger: mergedValidateTrigger
  }), function (control, meta, context) {
    var childNode = null;
    var isRequired = required !== undefined ? required : !!(rules && rules.some(function (rule) {
      if (rule && typeof rule === 'object' && rule.required) {
        return true;
      }

      return false;
    }));
    var fieldId = (utils_1.toArray(name).length && meta ? meta.name : []).join('_');

    if (isRenderProps && (props.shouldUpdate || props.dependencies) && !name) {
      childNode = children(context);
    } else if (react_1["default"].isValidElement(children)) {
      var childProps = _extends({}, children.props, control);

      if (!childProps.id) {
        childProps.id = fieldId;
      }

      if (disabled) {
        childProps.disabled = disabled;
      } // We should keep user origin event handler


      var triggers = new Set([].concat(utils_1.toArray(trigger), utils_1.toArray(mergedValidateTrigger)));
      triggers.forEach(function (eventName) {
        childProps[eventName] = function () {
          var _control$eventName, _children$props$event, _children$props;

          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          (_control$eventName = control[eventName]) == null ? void 0 : _control$eventName.call.apply(_control$eventName, [control].concat(args));
          (_children$props$event = (_children$props = children.props)[eventName]) == null ? void 0 : _children$props$event.call.apply(_children$props$event, [_children$props].concat(args));
        };
      });
      childNode = react_1["default"].createElement(MemoInput, {
        value: control[props.valuePropName || 'value'],
        update: updateRef.current
      }, react_1["default"].cloneElement(children, childProps));
    } else {
      childNode = children;
    }

    return renderLayout(childNode, fieldId, meta, isRequired);
  });
};