"use strict";

var __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {
  if (k2 === undefined) k2 = k;
  Object.defineProperty(o, k2, {
    enumerable: true,
    get: function get() {
      return m[k];
    }
  });
} : function (o, m, k, k2) {
  if (k2 === undefined) k2 = k;
  o[k2] = m[k];
});

var __setModuleDefault = this && this.__setModuleDefault || (Object.create ? function (o, v) {
  Object.defineProperty(o, "default", {
    enumerable: true,
    value: v
  });
} : function (o, v) {
  o["default"] = v;
});

var __importStar = this && this.__importStar || function (mod) {
  if (mod && mod.__esModule) return mod;
  var result = {};
  if (mod != null) for (var k in mod) {
    if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
  }

  __setModuleDefault(result, mod);

  return result;
};

var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};

Object.defineProperty(exports, "__esModule", {
  value: true
});

var react_1 = __importStar(require("react"));

var classnames_1 = __importDefault(require("classnames"));

var ticks_1 = __importDefault(require("./ticks"));

var marks_1 = __importDefault(require("./marks"));

var thumb_1 = __importDefault(require("./thumb"));

var ahooks_1 = require("ahooks");

var with_default_props_1 = require("../../utils/with-default-props");

var nearest_1 = require("../../utils/nearest");

var classPrefix = "am-slider";
var defaultProps = {
  min: 0,
  max: 100,
  step: 1,
  ticks: false,
  range: false,
  disabled: false
};
var Slider = with_default_props_1.withDefaultProps(defaultProps)(function (props) {
  var _classnames_1$default;

  var min = props.min,
      max = props.max,
      disabled = props.disabled,
      marks = props.marks,
      ticks = props.ticks,
      step = props.step;

  function sortValue(val) {
    return val.sort(function (a, b) {
      return a - b;
    });
  }

  function convertValue(value) {
    return props.range ? value : [props.min, value];
  }

  function reverseValue(value) {
    return props.range ? value : value[1];
  }

  function onAfterChange(value) {
    props.onAfterChange == null ? void 0 : props.onAfterChange(reverseValue(value));
  }

  var _ahooks_1$useControll = ahooks_1.useControllableValue(props, {
    valuePropName: 'value',
    defaultValuePropName: 'defaultValue',
    defaultValue: props.range ? [min, min] : min
  }),
      rawValue = _ahooks_1$useControll[0],
      setRawValue = _ahooks_1$useControll[1];

  var sliderValue = sortValue(convertValue(rawValue));

  function setSliderValue(value) {
    var next = sortValue(value);
    var current = sliderValue;
    if (next[0] === current[0] && next[1] === current[1]) return;
    setRawValue(reverseValue(next));
  }

  var trackRef = react_1.useRef(null);
  var fillSize = 100 * (sliderValue[1] - sliderValue[0]) / (max - min) + "%";
  var fillStart = 100 * (sliderValue[0] - min) / (max - min) + "%"; // 计算要显示的点

  var pointList = react_1.useMemo(function () {
    if (marks) {
      return Object.keys(marks).map(parseFloat).sort(function (a, b) {
        return a - b;
      });
    } else {
      var points = [];

      for (var i = min; i <= max; i += step) {
        points.push(i);
      }

      return points;
    }
  }, [marks, ticks, step, min, max]);

  function getValueByPosition(position) {
    var newPosition = position < min ? min : position > max ? max : position;
    var value = min; // 显示了刻度点，就只能移动到点上

    if (pointList.length) {
      value = nearest_1.nearest(pointList, newPosition);
    } else {
      var lengthPerStep = 100 / ((max - min) / step);
      var steps = Math.round(newPosition / lengthPerStep);
      value = steps * lengthPerStep * (max - min) * 0.01 + min;
    }

    return value;
  }

  var dragLockRef = react_1.useRef(0);

  var onTrackClick = function onTrackClick(event) {
    if (dragLockRef.current > 0) return;
    event.stopPropagation();
    if (disabled) return;
    var sliderOffsetLeft = trackRef.current.getBoundingClientRect().left;
    var position = (event.clientX - sliderOffsetLeft) / Math.ceil(trackRef.current.offsetWidth) * (max - min) + min;
    var targetValue = getValueByPosition(position);
    var nextSliderValue;

    if (props.range) {
      // 移动的滑块采用就近原则
      if (Math.abs(targetValue - sliderValue[0]) > Math.abs(targetValue - sliderValue[1])) {
        nextSliderValue = [sliderValue[0], targetValue];
      } else {
        nextSliderValue = [targetValue, sliderValue[1]];
      }
    } else {
      nextSliderValue = [props.min, targetValue];
    }

    setSliderValue(nextSliderValue);
    onAfterChange(nextSliderValue);
  };

  var valueBeforeDragRef = react_1.useRef();

  var renderThumb = function renderThumb(index) {
    return react_1["default"].createElement(thumb_1["default"], {
      key: index,
      value: sliderValue[index],
      min: min,
      max: max,
      disabled: disabled,
      trackRef: trackRef,
      onDrag: function onDrag(position, first, last) {
        if (first) {
          dragLockRef.current += 1;
          valueBeforeDragRef.current = sliderValue;
        }

        var val = getValueByPosition(position);
        var valueBeforeDrag = valueBeforeDragRef.current;
        if (!valueBeforeDrag) return;
        var next = [].concat(valueBeforeDrag);
        next[index] = val;
        setSliderValue(next);

        if (last) {
          onAfterChange(next);
          window.setTimeout(function () {
            dragLockRef.current -= 1;
          }, 100);
        }
      }
    });
  };

  return react_1["default"].createElement("div", {
    className: classnames_1["default"](classPrefix, (_classnames_1$default = {}, _classnames_1$default[classPrefix + "-disabled"] = disabled, _classnames_1$default), props.className),
    style: props.style
  }, react_1["default"].createElement("div", {
    className: classPrefix + "-track-container",
    onClick: onTrackClick
  }, react_1["default"].createElement("div", {
    className: classPrefix + "-track",
    onClick: onTrackClick,
    ref: trackRef
  }, react_1["default"].createElement("div", {
    className: classPrefix + "-fill",
    style: {
      width: fillSize,
      left: fillStart
    }
  }), props.ticks && react_1["default"].createElement(ticks_1["default"], {
    points: pointList,
    min: min,
    max: max,
    lowerBound: sliderValue[0],
    upperBound: sliderValue[1]
  }), props.range && renderThumb(0), renderThumb(1))), marks && react_1["default"].createElement(marks_1["default"], {
    min: min,
    max: max,
    marks: marks,
    lowerBound: sliderValue[0],
    upperBound: sliderValue[1]
  }));
});
exports["default"] = Slider;