"use strict";

var _excluded = ["className", "style", "defaultValue", "value", "onChange", "rows", "autoSize", "showCount"];

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

var __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {
  if (k2 === undefined) k2 = k;
  Object.defineProperty(o, k2, {
    enumerable: true,
    get: function get() {
      return m[k];
    }
  });
} : function (o, m, k, k2) {
  if (k2 === undefined) k2 = k;
  o[k2] = m[k];
});

var __setModuleDefault = this && this.__setModuleDefault || (Object.create ? function (o, v) {
  Object.defineProperty(o, "default", {
    enumerable: true,
    value: v
  });
} : function (o, v) {
  o["default"] = v;
});

var __importStar = this && this.__importStar || function (mod) {
  if (mod && mod.__esModule) return mod;
  var result = {};
  if (mod != null) for (var k in mod) {
    if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
  }

  __setModuleDefault(result, mod);

  return result;
};

var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};

Object.defineProperty(exports, "__esModule", {
  value: true
});

var react_1 = __importStar(require("react"));

var classnames_1 = __importDefault(require("classnames"));

var ahooks_1 = require("ahooks");

var classPrefix = 'am-text-area';
var defaultProps = {
  rows: 2,
  showCount: false,
  autoSize: false
}; // TODO: withDefaultProps 和 forwardRef 配合使用的问题

var TextArea = react_1.forwardRef(function (props, ref) {
  var className = props.className,
      style = props.style,
      outerDefaultValue = props.defaultValue,
      outerValue = props.value,
      outerOnChange = props.onChange,
      rows = props.rows,
      autoSize = props.autoSize,
      showCount = props.showCount,
      textAreaProps = _objectWithoutPropertiesLoose(props, _excluded);

  var _ahooks_1$useControll = ahooks_1.useControllableValue(props, {
    defaultValue: ''
  }),
      value = _ahooks_1$useControll[0],
      setValue = _ahooks_1$useControll[1];

  var nativeTextAreaRef = react_1.useRef(null);
  react_1.useImperativeHandle(ref, function () {
    return {
      clear: function clear() {
        setValue('');
      },
      focus: function focus() {
        var _nativeTextAreaRef$cu;

        (_nativeTextAreaRef$cu = nativeTextAreaRef.current) == null ? void 0 : _nativeTextAreaRef$cu.focus();
      },
      blur: function blur() {
        var _nativeTextAreaRef$cu2;

        (_nativeTextAreaRef$cu2 = nativeTextAreaRef.current) == null ? void 0 : _nativeTextAreaRef$cu2.blur();
      }
    };
  });
  react_1.useEffect(function () {
    if (!autoSize) return;
    var textArea = nativeTextAreaRef.current;
    if (!textArea) return;
    textArea.style.height = 'auto';
    var height = textArea.scrollHeight;

    if (typeof autoSize === 'object') {
      var computedStyle = window.getComputedStyle(textArea);
      var lineHeight = parseFloat(computedStyle.lineHeight);

      if (autoSize.minRows) {
        height = Math.max(height, autoSize.minRows * lineHeight);
      }

      if (autoSize.maxRows) {
        height = Math.min(height, autoSize.maxRows * lineHeight);
      }
    }

    textArea.style.height = height + "px";
  }, [value, autoSize]);
  return react_1["default"].createElement("div", {
    className: classnames_1["default"](classPrefix + "-wrapper", className),
    style: style
  }, react_1["default"].createElement("textarea", Object.assign({
    ref: nativeTextAreaRef
  }, textAreaProps, {
    className: classPrefix,
    rows: rows,
    value: value,
    onChange: function onChange(e) {
      setValue(e.target.value);
    },
    onFocus: function onFocus(e) {
      props.onFocus == null ? void 0 : props.onFocus(e);
    },
    onBlur: function onBlur(e) {
      props.onBlur == null ? void 0 : props.onBlur(e);
    }
  })), showCount && react_1["default"].createElement("div", {
    className: classPrefix + "-count"
  }, props.maxLength === undefined ? value.length : value.length + '/' + props.maxLength));
});
TextArea.defaultProps = defaultProps;
exports["default"] = TextArea;