import React, { forwardRef, useState, useImperativeHandle, createRef, useEffect } from 'react';
import { withDefaultProps } from '../../utils/with-default-props';
import classNames from 'classnames';
import Popup from '../popup';
import Button from '../button';
import { attachPropertiesToComponent } from '../../utils/attach-properties-to-component';
import { renderToBody } from '../../utils/render-to-body';
var classPrefix = "am-action-sheet";
var defaultProps = {
  visible: false,
  actions: [],
  cancelText: '',
  closeOnSelect: false,
  closeOnMaskClick: true
};
var ActionSheet = withDefaultProps(defaultProps)(function (props) {
  return /*#__PURE__*/React.createElement(Popup, {
    visible: props.visible,
    onMaskClick: function onMaskClick() {
      props.onMaskClick == null ? void 0 : props.onMaskClick();

      if (props.closeOnMaskClick) {
        props.onClose == null ? void 0 : props.onClose();
      }
    },
    afterClose: props.afterClose,
    className: classPrefix + "-popup",
    getContainer: props.getContainer
  }, /*#__PURE__*/React.createElement("div", {
    className: classNames(classPrefix, props.className)
  }, props.extra && /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-extra"
  }, props.extra), /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-button-list"
  }, props.actions.map(function (action, index) {
    var _classNames;

    return /*#__PURE__*/React.createElement("div", {
      key: action.key,
      className: classPrefix + "-button-item-wrapper"
    }, /*#__PURE__*/React.createElement(Button, {
      block: true,
      fill: 'none',
      disabled: action.disabled,
      onClick: function onClick() {
        action.onClick == null ? void 0 : action.onClick();
        props.onSelect == null ? void 0 : props.onSelect(action, index);

        if (props.closeOnSelect) {
          props.onClose == null ? void 0 : props.onClose();
        }
      },
      className: classNames(classPrefix + "-button-item", (_classNames = {}, _classNames[classPrefix + "-button-item-danger"] = action.danger, _classNames))
    }, /*#__PURE__*/React.createElement("div", {
      className: classPrefix + "-button-item-name"
    }, action.text), action.description && /*#__PURE__*/React.createElement("div", {
      className: classPrefix + "-button-item-description"
    }, action.description)));
  })), props.cancelText && /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-cancel"
  }, /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-button-item-wrapper"
  }, /*#__PURE__*/React.createElement(Button, {
    block: true,
    fill: 'none',
    className: classPrefix + "-button-item",
    onClick: function onClick() {
      props.onClose == null ? void 0 : props.onClose();
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-button-item-name"
  }, props.cancelText))))));
});

function showActionSheet(props) {
  var Wrapper = /*#__PURE__*/forwardRef(function (_, ref) {
    var _useState = useState(false),
        visible = _useState[0],
        setVisible = _useState[1];

    useEffect(function () {
      setVisible(true);
    }, []);
    useImperativeHandle(ref, function () {
      return {
        close: function close() {
          setVisible(false);
        }
      };
    });
    return /*#__PURE__*/React.createElement(ActionSheet, Object.assign({}, props, {
      visible: visible,
      onClose: function onClose() {
        props.onClose == null ? void 0 : props.onClose();
        setVisible(false);
      },
      afterClose: function afterClose() {
        props.afterClose == null ? void 0 : props.afterClose();
        unmount();
      }
    }));
  });
  var ref = /*#__PURE__*/createRef();
  var unmount = renderToBody( /*#__PURE__*/React.createElement(Wrapper, {
    ref: ref
  }));
  return {
    close: function close() {
      var _ref$current;

      (_ref$current = ref.current) == null ? void 0 : _ref$current.close();
    }
  };
}

export default attachPropertiesToComponent(ActionSheet, {
  show: showActionSheet
});