import { useControllableValue } from 'ahooks';
import classNames from 'classnames';
import React, { useMemo } from 'react';
import { attachPropertiesToComponent } from '../../utils/attach-properties-to-component';
import { getTreeDeep } from '../../utils/tree';
import { withDefaultProps } from '../../utils/with-default-props';
import Multiple from './multiple';
var classPrefix = "am-cascader";
var Cascader = withDefaultProps({
  options: [],
  fieldNames: {}
})(function (props) {
  var labelName = props.fieldNames.label || 'label';
  var valueName = props.fieldNames.value || 'value';
  var childrenName = props.fieldNames.children || 'children';

  var _useControllableValue = useControllableValue(props, {
    defaultValue: []
  }),
      value = _useControllableValue[0],
      setValue = _useControllableValue[1];

  var _useMemo = useMemo(function () {
    var deep = getTreeDeep(props.options, childrenName);
    var optionsMap = new Map();
    var optionsParentMap = new Map();

    function traverse(current, children) {
      children.forEach(function (item) {
        optionsParentMap.set(item[valueName], current);
        optionsMap.set(item[valueName], item);

        if (item[childrenName]) {
          traverse(item, item[childrenName]);
        }
      });
    }

    traverse(undefined, props.options);
    return [deep, optionsMap, optionsParentMap];
  }, [props.options]),
      deep = _useMemo[0],
      optionsMap = _useMemo[1],
      optionsParentMap = _useMemo[2];

  var onItemSelect = function onItemSelect(node) {
    // 找到父级节点
    var parentNodes = [];
    var current = node;

    while (current) {
      parentNodes.unshift(current);
      var next = optionsParentMap.get(current[valueName]);
      current = next;
    }

    var values = parentNodes.map(function (i) {
      return i[valueName];
    });
    setValue(values, parentNodes);
  };

  var renderItems = function renderItems(columnOptions, index) {
    if (columnOptions === void 0) {
      columnOptions = [];
    }

    return columnOptions.map(function (item) {
      var _classNames;

      var isActive = item[valueName] === value[index];
      return /*#__PURE__*/React.createElement("div", {
        key: item[valueName],
        className: classNames(classPrefix + "-item", (_classNames = {}, _classNames[classPrefix + "-item-active"] = isActive, _classNames)),
        onClick: function onClick() {
          if (!isActive) {
            onItemSelect(item);
          }
        }
      }, item[labelName]);
    });
  };

  var renderColumns = function renderColumns() {
    var columns = [];

    for (var i = 0; i < deep; i++) {
      var _optionsMap$get;

      var width = 100 / deep + "%"; // 两列的第一列宽度为 33.33，两列的第二列为 66.67%

      if (deep === 2 && i === 0) {
        width = "33.33%";
      }

      if (deep === 2 && i === 1) {
        width = "66.67%";
      }

      var column = /*#__PURE__*/React.createElement("div", {
        key: i,
        className: classNames(classPrefix + "-column"),
        style: {
          width: width
        }
      }, renderItems(i === 0 ? props.options : (_optionsMap$get = optionsMap.get(value[i - 1])) == null ? void 0 : _optionsMap$get[childrenName], i));
      columns.push(column);
    }

    return columns;
  };

  return /*#__PURE__*/React.createElement("div", {
    className: classNames(classPrefix, props.className),
    style: props.style
  }, renderColumns());
});
export default attachPropertiesToComponent(Cascader, {
  Multiple: Multiple
});