import React, { useEffect, useRef } from 'react';
import { useControllableValue } from 'ahooks';
import { attachPropertiesToComponent } from '../../utils/attach-properties-to-component';
import List from '../list';
import { RightOutlined } from '@ant-design/icons';
import classNames from 'classnames';
import { useInitialized } from '../../utils/use-initialized';
import { useSpring, animated } from '@react-spring/web';

var CollapsePanel = function CollapsePanel() {
  return null;
};

var CollapsePanelContent = function CollapsePanelContent(props) {
  var visible = props.visible;
  var innerRef = useRef(null);
  var initialized = useInitialized(visible || props.forceRender);

  var _useSpring = useSpring(function () {
    return {
      from: {
        height: visible ? 'auto' : 0
      }
    };
  }),
      style = _useSpring[0],
      api = _useSpring[1];

  useEffect(function () {
    if (visible) {
      var inner = innerRef.current;
      if (!inner) return;
      api.start({
        height: inner.offsetHeight
      });
    } else {
      api.start({
        height: 0
      });
    }
  }, [visible]);
  return initialized ? /*#__PURE__*/React.createElement(animated.div, {
    className: 'am-collapse-panel-content',
    style: style
  }, /*#__PURE__*/React.createElement("div", {
    className: 'am-collapse-panel-content-inner',
    ref: innerRef
  }, /*#__PURE__*/React.createElement(List.Item, null, props.children))) : null;
};

var Collapse = function Collapse(props) {
  var panels = [];
  React.Children.forEach(props.children, function (child) {
    if (! /*#__PURE__*/React.isValidElement(child)) return;
    var key = child.key;
    if (typeof key !== 'string') return;
    panels.push(child);
  });

  var _useControllableValue = useControllableValue(props, {
    valuePropName: 'activeKey',
    defaultValuePropName: 'defaultActiveKey',
    defaultValue: [],
    trigger: 'onChange'
  }),
      activeKey = _useControllableValue[0],
      setActiveKey = _useControllableValue[1];

  var activeKeyList = activeKey === null ? [] : Array.isArray(activeKey) ? activeKey : [activeKey];
  return /*#__PURE__*/React.createElement("div", {
    className: classNames('am-collapse', props.className),
    style: props.style
  }, /*#__PURE__*/React.createElement(List, null, panels.map(function (panel) {
    var key = panel.key;
    var active = activeKeyList.includes(key);

    function handleClick() {
      if (props.accordion) {
        if (active) {
          setActiveKey(null);
        } else {
          setActiveKey(key);
        }
      } else {
        if (active) {
          setActiveKey(activeKeyList.filter(function (v) {
            return v !== key;
          }));
        } else {
          setActiveKey([].concat(activeKeyList, [key]));
        }
      }
    }

    return /*#__PURE__*/React.createElement(React.Fragment, {
      key: key
    }, /*#__PURE__*/React.createElement(List.Item, {
      className: classNames('am-collapse-panel-header', {
        'am-collapse-panel-header-disabled': panel.props.disabled
      }),
      onClick: panel.props.disabled ? undefined : handleClick,
      arrow: /*#__PURE__*/React.createElement("div", {
        className: classNames('am-collapse-arrow', {
          'am-collapse-arrow-active': active
        })
      }, /*#__PURE__*/React.createElement(RightOutlined, null))
    }, panel.props.title), /*#__PURE__*/React.createElement(CollapsePanelContent, {
      visible: active,
      forceRender: !!panel.props.forceRender
    }, panel.props.children));
  })));
};

export default attachPropertiesToComponent(Collapse, {
  Panel: CollapsePanel
});