import React from 'react';
export interface DialogBtnProps {
    loading?: boolean;
    disabled?: boolean;
}
export interface DialogProps {
    /** Dialog 完全关闭后的回调 */
    afterClose?: () => void;
    /** 顶部图片 url */
    headerImage?: string;
    /** 是否等待图片加载完毕后再弹出对话框，默认为 true */
    waitImageLoad?: boolean;
    /** Dialog 内容样式 */
    bodyStyle?: React.CSSProperties;
    /** Dialog 内容类名 */
    bodyClassName?: string;
    /** Dialog 遮罩样式 */
    maskStyle?: React.CSSProperties;
    /** Dialog 遮罩类名 */
    maskClassName?: string;
    /** 对话框标题 */
    title?: React.ReactNode;
    /** 对话框提示文字 */
    content?: React.ReactNode;
    /** 取消的文字 */
    cancelText?: React.ReactNode;
    /** 点击取消的回调 */
    onCancel?: (e: React.MouseEvent) => void | boolean | Promise<boolean> | Promise<void> | Promise<void | boolean>;
    /** 取消按钮的状态 */
    cancelProps?: DialogBtnProps;
    /** 确认的文字 */
    okText?: React.ReactNode;
    /** 确认按钮的状态 */
    okProps?: DialogBtnProps;
    /** 点击确认的回调 */
    onOk?: (e: React.MouseEvent) => void | boolean | Promise<boolean> | Promise<void> | Promise<void | boolean>;
    /** 是否支持点击遮罩关闭对话框 */
    maskClosable?: boolean;
    /** 是否展示 */
    visible?: boolean;
    /** 自定义挂载父容器、默认为 document.body */
    getContainer?: HTMLElement | (() => HTMLElement) | undefined;
}
export declare type AlertProps = Omit<DialogProps, 'okText' | 'okProps' | 'onOk'>;
declare type DialogType = {
    show: (props: DialogProps) => void;
    confirm: (props: DialogProps) => Promise<boolean>;
    alert: (props: DialogProps) => Promise<React.MouseEvent>;
};
declare const Dialog: DialogType;
export default Dialog;
