import React, { useRef, useState, useLayoutEffect } from 'react';
import classNames from 'classnames';
import { withDefaultProps } from '../../utils/with-default-props';
var classPrefix = "am-ellipsis";
var defaultProps = {
  direction: 'end',
  rows: 1
};
var Ellipsis = withDefaultProps(defaultProps)(function (props) {
  var originRef = useRef(null);

  var _useState = useState(''),
      ellipsised = _useState[0],
      setEllipsised = _useState[1];

  useLayoutEffect(function () {
    var origin = originRef.current;
    if (!origin) return;
    var originStyle = window.getComputedStyle(origin);
    var container = document.createElement('div');
    var styleNames = Array.prototype.slice.apply(originStyle);
    styleNames.forEach(function (name) {
      container.style.setProperty(name, originStyle.getPropertyValue(name));
    });
    container.style.position = 'fixed';
    container.style.left = '999999px';
    container.style.top = '999999px';
    container.style.zIndex = '-1000';
    container.style.height = 'auto';
    container.style.minHeight = 'auto';
    container.style.maxHeight = 'auto';
    container.style.textOverflow = 'clip';
    container.style.whiteSpace = 'normal';
    container.style.webkitLineClamp = 'unset';
    container.style.webkitBoxOrient = 'unset';
    container.style.display = 'block';
    var lineHeight = pxToNumber(originStyle.lineHeight);
    var maxHeight = Math.floor(lineHeight) * props.rows + pxToNumber(originStyle.paddingTop) + pxToNumber(originStyle.paddingBottom);
    container.innerText = props.content;
    document.body.appendChild(container);

    if (container.offsetHeight <= maxHeight) {
      setEllipsised(props.content);
    } else {
      var check = function check(left, right) {
        if (right - left <= 1) {
          if (props.direction === 'end') {
            return props.content.slice(0, left) + '...';
          } else {
            return '...' + props.content.slice(right, end);
          }
        }

        var middle = Math.round((left + right) / 2);

        if (props.direction === 'end') {
          container.innerText = props.content.slice(0, middle) + '...';
        } else {
          container.innerText = '...' + props.content.slice(middle, end);
        }

        if (container.offsetHeight <= maxHeight) {
          if (props.direction === 'end') {
            return check(middle, right);
          } else {
            return check(left, middle);
          }
        } else {
          if (props.direction === 'end') {
            return check(left, middle);
          } else {
            return check(middle, right);
          }
        }
      };

      var end = props.content.length;
      setEllipsised(check(0, props.content.length));
    }

    document.body.removeChild(container);
  }, [props.content, props.rows, props.direction]);
  return /*#__PURE__*/React.createElement("div", {
    ref: originRef,
    className: classNames(classPrefix, props.className),
    style: props.style
  }, ellipsised);
});

function pxToNumber(value) {
  if (!value) return 0;
  var match = value.match(/^\d*(\.\d*)?/);
  return match ? Number(match[0]) : 0;
}

export default Ellipsis;