import { withDefaultProps } from '../../utils/with-default-props';
import React, { useEffect, useRef } from 'react';
import { useLockFn, usePersistFn } from 'ahooks';
import classNames from 'classnames';
import { getScrollParent } from '../../utils/get-scroll-parent';
import Loading from '../loading';

function _empty() {}

function _awaitIgnored(value, direct) {
  if (!direct) {
    return value && value.then ? value.then(_empty) : Promise.resolve();
  }
}

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function isWindow(element) {
  return element === window;
}

var classPrefix = "am-infinite-scroll";

var InfiniteScrollContent = function InfiniteScrollContent(_ref) {
  var hasMore = _ref.hasMore;
  return /*#__PURE__*/React.createElement(React.Fragment, null, hasMore ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("span", null, "\u52A0\u8F7D\u4E2D"), /*#__PURE__*/React.createElement(Loading, {
    size: 'small'
  })) : /*#__PURE__*/React.createElement("span", null, "\u6CA1\u6709\u66F4\u591A\u4E86"));
};

var InfiniteScroll = withDefaultProps({
  threshold: 250
})(function (props) {
  var doLoadMore = useLockFn(_async(function () {
    return _awaitIgnored(props.loadMore());
  }));
  var elementRef = useRef(null);
  var check = usePersistFn(function () {
    if (!props.hasMore) return;
    var element = elementRef.current;
    if (!element) return;
    var parent = getScrollParent(element);
    if (!parent) return;
    var elementTop = element.getBoundingClientRect().top;
    var current = isWindow(parent) ? window.innerHeight : parent.getBoundingClientRect().bottom;

    if (current >= elementTop - props.threshold) {
      doLoadMore();
    }
  }); // 确保在内容不足时会自动触发加载事件

  useEffect(function () {
    check();
  });
  useEffect(function () {
    var element = elementRef.current;
    if (!element) return;
    var parent = getScrollParent(element);
    if (!parent) return;

    function onScroll() {
      check();
    }

    parent.addEventListener('scroll', onScroll);
    return function () {
      parent.removeEventListener('scroll', onScroll);
    };
  }, []);
  return /*#__PURE__*/React.createElement("div", {
    className: classNames(classPrefix, props.className),
    style: props.style,
    ref: elementRef
  }, props.children && props.children, !props.children && /*#__PURE__*/React.createElement(InfiniteScrollContent, {
    hasMore: props.hasMore
  }));
});
export default InfiniteScroll;