function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import { useInitialized } from '../../utils/use-initialized';
import { disableBodyScroll, enableBodyScroll } from 'body-scroll-lock';
import classNames from 'classnames';
import React, { useEffect, useRef } from 'react';
import { createPortal } from 'react-dom';
import { CSSTransition } from 'react-transition-group';
var classPrefix = "am-mask";

var Mask = function Mask(props) {
  var _classNames;

  var cls = classNames(classPrefix, props.className, (_classNames = {}, _classNames[classPrefix + "-hidden"] = !props.visible, _classNames));
  var initialized = useInitialized(props.visible || props.forceRender);
  var ref = useRef(null); // 禁止 body 滚动 https://github.com/willmcpo/body-scroll-lock

  useEffect(function () {
    var element = ref.current;

    if (props.visible && props.disableBodyScroll) {
      disableBodyScroll(element);
      return function () {
        enableBodyScroll(element);
      };
    }
  }, [props.visible, props.disableBodyScroll]);

  function handleClick(e) {
    if (e.currentTarget === e.target) {
      props.onMaskClick == null ? void 0 : props.onMaskClick(e);
    }
  }

  var opacity = props.opacity === 'default' ? 0.55 : props.opacity === 'dark' ? 0.75 : props.opacity;
  var node = /*#__PURE__*/React.createElement(CSSTransition, {
    "in": props.visible,
    timeout: 200,
    classNames: classPrefix,
    onExited: props.afterClose,
    unmountOnExit: props.destroyOnClose
  }, /*#__PURE__*/React.createElement("div", {
    className: cls,
    onClick: handleClick,
    ref: ref,
    style: _extends({}, props.style, {
      backgroundColor: "rgba(0, 0, 0, " + opacity + ")"
    })
  }, initialized && props.children));

  if (props.getContainer) {
    var container = typeof props.getContainer === 'function' ? props.getContainer() : props.getContainer;
    return /*#__PURE__*/createPortal(node, container);
  }

  return node;
};

Mask.defaultProps = {
  visible: true,
  destroyOnClose: false,
  forceRender: false,
  opacity: 'default',
  disableBodyScroll: true
};
export default Mask;