import React, { useState, useRef, useLayoutEffect, memo } from 'react';
import classNames from 'classnames';
import { CloseOutlined, SoundOutlined } from '@ant-design/icons';
import { usePersistFn, useUpdateLayoutEffect } from 'ahooks';
import { noop } from '../../utils/noop';
import { withDefaultProps } from '../../utils/with-default-props';
var classPrefix = "am-notice-bar";
var defaultProps = {
  delay: 2000,
  speed: 50
};
var NoticeBar = /*#__PURE__*/memo(withDefaultProps(defaultProps)(function (props) {
  var containerRef = useRef(null);
  var textRef = useRef(null);

  var _useState = useState(0),
      key = _useState[0],
      setKey = _useState[1];

  var onClose = usePersistFn(props.onClose || noop);
  var speed = props.speed;
  useLayoutEffect(function () {
    var container = containerRef.current;
    var text = textRef.current;
    if (!container || !text) return;
    if (container.offsetWidth >= text.offsetWidth) return; // 需要滚动

    var timeout = window.setTimeout(function () {
      var text = textRef.current; // 开始滚动

      if (text) {
        text.style.transitionDuration = Math.round(text.offsetWidth / speed) + "s";
        text.style.transform = "translateX(-" + text.offsetWidth + "px)";
      }
    }, props.delay);
    return function () {
      window.clearTimeout(timeout);
    };
  }, []);
  useUpdateLayoutEffect(function () {
    var container = containerRef.current;
    var text = textRef.current;
    if (!container || !text) return;
    if (container.offsetWidth >= text.offsetWidth) return;
    text.style.transform = "translateX(" + container.offsetWidth + "px)";
    text.style.transitionDuration = Math.round((container.offsetWidth + text.offsetWidth) / speed) + "s";
    text.style.transform = "translateX(-" + text.offsetWidth + "px)";
  }, [key]);
  return /*#__PURE__*/React.createElement("div", {
    className: classNames(classPrefix, classPrefix + "-" + (props.type || 'default'))
  }, /*#__PURE__*/React.createElement("span", {
    className: classPrefix + "-left"
  }, 'icon' in props ? props.icon : /*#__PURE__*/React.createElement(SoundOutlined, null)), /*#__PURE__*/React.createElement("span", {
    ref: containerRef,
    className: classPrefix + "-content"
  }, /*#__PURE__*/React.createElement("span", {
    onTransitionEnd: function onTransitionEnd() {
      return setKey(function (k) {
        return k + 1;
      });
    },
    key: key,
    ref: textRef,
    className: classPrefix + "-content-inner"
  }, props.content)), (props.closeable || 'extra' in props) && /*#__PURE__*/React.createElement("span", {
    className: classPrefix + "-right"
  }, props.extra, props.closeable && /*#__PURE__*/React.createElement(CloseOutlined, {
    onClick: onClose
  })));
}));
export default NoticeBar;