import React, { useState, useEffect, useMemo } from 'react';
import Popup from '../popup';
import { useControllableValue } from 'ahooks';
import { Column } from './column';
import { withDefaultProps } from '../../utils/with-default-props';
import { attachPropertiesToComponent } from '../../utils/attach-properties-to-component';
import { Cascader } from './cascader';
import classNames from 'classnames';
import { renderToBody } from '../../utils/render-to-body';
var classPrefix = "am-picker";
var Picker = withDefaultProps({
  confirmText: '确定',
  cancelText: '取消'
})(function (props) {
  var controllable = useControllableValue(props, {
    trigger: 'onConfirm',
    defaultValue: []
  });
  var value = controllable[0];
  var setValue = controllable[1];

  var _useState = useState(value),
      innerValue = _useState[0],
      setInnerValue = _useState[1];

  useEffect(function () {
    setInnerValue(value);
  }, [props.visible]);
  var columns = useMemo(function () {
    var columns = typeof props.columns === 'function' ? props.columns(innerValue) : props.columns;
    return columns.map(function (column) {
      return column.map(function (item) {
        return typeof item === 'string' ? {
          label: item,
          value: item
        } : item;
      });
    });
  }, [props.columns, innerValue]);
  var widget = /*#__PURE__*/React.createElement(Popup, {
    visible: props.visible,
    position: 'bottom',
    onMaskClick: function onMaskClick() {
      props.onCancel == null ? void 0 : props.onCancel();
      props.onClose == null ? void 0 : props.onClose();
    },
    getContainer: props.getContainer,
    destroyOnClose: true,
    afterShow: props.afterShow,
    afterClose: props.afterClose
  }, /*#__PURE__*/React.createElement("div", {
    className: classNames(classPrefix, props.className),
    style: props.style
  }, /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-header"
  }, /*#__PURE__*/React.createElement("a", {
    className: classPrefix + "-header-button",
    onClick: function onClick() {
      props.onCancel == null ? void 0 : props.onCancel();
      props.onClose == null ? void 0 : props.onClose();
    }
  }, props.cancelText), /*#__PURE__*/React.createElement("a", {
    className: classPrefix + "-header-button",
    onClick: function onClick() {
      setValue(innerValue);
      props.onClose == null ? void 0 : props.onClose();
    }
  }, props.confirmText)), /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-columns"
  }, columns.map(function (column, index) {
    return /*#__PURE__*/React.createElement(Column, {
      key: index,
      column: column,
      value: innerValue[index],
      onSelect: function onSelect(val) {
        setInnerValue(function (prev) {
          var nextValue = [].concat(prev);
          nextValue[index] = val;
          props.onSelect == null ? void 0 : props.onSelect(nextValue);
          return nextValue;
        });
      }
    });
  }), /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-mask " + classPrefix + "-mask-top"
  }), /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-mask " + classPrefix + "-mask-bottom"
  }))));
  var label = useMemo(function () {
    if (!props.children) return null;
    var items = columns.map(function (column, index) {
      var _column$find;

      return (_column$find = column.find(function (item) {
        return item.value === value[index];
      })) != null ? _column$find : null;
    });
    return props.children(items);
  }, [value]);
  return /*#__PURE__*/React.createElement(React.Fragment, null, widget, label);
});

function prompt(props) {
  return new Promise(function (resolve) {
    var Wrapper = function Wrapper() {
      var _useState2 = useState(false),
          visible = _useState2[0],
          setVisible = _useState2[1];

      useEffect(function () {
        setVisible(true);
      }, []);
      return /*#__PURE__*/React.createElement(Picker, Object.assign({}, props, {
        visible: visible,
        onConfirm: function onConfirm(val) {
          resolve(val);
        },
        onClose: function onClose() {
          props.onClose == null ? void 0 : props.onClose();
          setVisible(false);
          resolve(null);
        },
        afterClose: function afterClose() {
          props.afterClose == null ? void 0 : props.afterClose();
          unmount();
        }
      }));
    };

    var unmount = renderToBody( /*#__PURE__*/React.createElement(Wrapper, null));
  });
}

function promptCascader(props) {
  return new Promise(function (resolve) {
    var Wrapper = function Wrapper() {
      var _useState3 = useState(false),
          visible = _useState3[0],
          setVisible = _useState3[1];

      useEffect(function () {
        setVisible(true);
      }, []);
      return /*#__PURE__*/React.createElement(Cascader, Object.assign({}, props, {
        visible: visible,
        onConfirm: function onConfirm(val) {
          resolve(val);
        },
        onClose: function onClose() {
          props.onClose == null ? void 0 : props.onClose();
          setVisible(false);
          resolve(null);
        },
        afterClose: function afterClose() {
          props.afterClose == null ? void 0 : props.afterClose();
          unmount();
        }
      }));
    };

    var unmount = renderToBody( /*#__PURE__*/React.createElement(Wrapper, null));
  });
}

export default attachPropertiesToComponent(Picker, {
  Cascader: attachPropertiesToComponent(Cascader, {
    prompt: promptCascader
  }),
  prompt: prompt
});