import React, { useMemo, useCallback, useRef, forwardRef, useImperativeHandle } from 'react';
import Tooltip from 'rc-tooltip';
import { useControllableValue } from 'ahooks';
import Button from '../button';
import classNames from 'classnames';
import { attachPropertiesToComponent } from '../../utils/attach-properties-to-component';
var classPrefix = "am-popover";
var enterClassName = 'entering';
var leaveClassName = 'leaving';
var Popover = /*#__PURE__*/forwardRef(function (props, ref) {
  var _props$mode = props.mode,
      mode = _props$mode === void 0 ? 'light' : _props$mode;

  var _useControllableValue = useControllableValue(props, {
    valuePropName: 'visible',
    trigger: 'onVisibleChange',
    defaultValuePropName: 'defaultVisible',
    defaultValue: false
  }),
      visible = _useControllableValue[0],
      onVisibleChange = _useControllableValue[1];

  useImperativeHandle(ref, function () {
    return {
      show: function show() {
        return onVisibleChange(true);
      },
      hide: function hide() {
        return onVisibleChange(false);
      },
      visible: visible
    };
  }, [visible]);
  return /*#__PURE__*/React.createElement(Tooltip, Object.assign({}, props, {
    overlayClassName: classNames(classPrefix + "-" + mode, props.overlayClassName),
    destroyTooltipOnHide: props.destroyOnHide,
    prefixCls: classPrefix,
    getTooltipContainer: props.getContainer || function () {
      return document.body;
    },
    visible: visible,
    onVisibleChange: onVisibleChange,
    trigger: 'click',
    motion: {
      motionName: {
        appear: enterClassName,
        appearActive: enterClassName,
        enter: enterClassName,
        enterActive: enterClassName,
        leaveActive: leaveClassName,
        leave: leaveClassName
      },
      motionDeadline: 200
    },
    overlay: /*#__PURE__*/React.createElement("div", {
      className: classPrefix + "-inner-content"
    }, props.content)
  }), props.children);
});
var PopMenu = /*#__PURE__*/forwardRef(function (props, ref) {
  var innerRef = useRef(null);
  useImperativeHandle(ref, function () {
    return innerRef.current;
  }, []);

  var _onClick = useCallback(function (e) {
    var _innerRef$current;

    var onSelect = props.onSelect;

    if (onSelect) {
      onSelect(e);
    }

    (_innerRef$current = innerRef.current) == null ? void 0 : _innerRef$current.hide();
  }, [props.onSelect]);

  var overlay = useMemo(function () {
    return /*#__PURE__*/React.createElement(React.Fragment, null, (props.actions || []).map(function (ele, index) {
      var _classNames, _ele$key;

      return /*#__PURE__*/React.createElement("div", {
        className: classNames(classPrefix + "-inner-menu", (_classNames = {}, _classNames[classPrefix + "-inner-menu-with-icon"] = !!ele.icon, _classNames)),
        key: (_ele$key = ele.key) != null ? _ele$key : index
      }, /*#__PURE__*/React.createElement(Button, {
        disabled: ele.disabled,
        onClick: function onClick() {
          if (!ele.disabled) {
            _onClick(ele);
          }
        },
        fill: 'none',
        block: true
      }, ele.icon && /*#__PURE__*/React.createElement("span", {
        className: classPrefix + "-inner-menu-icon"
      }, ele.icon), ele.text));
    }));
  }, [props.actions, _onClick]);
  return /*#__PURE__*/React.createElement(Popover, Object.assign({
    ref: innerRef
  }, props, {
    overlayClassName: classNames(classPrefix + "-menu", props.overlayClassName),
    content: overlay
  }), props.children);
});
export default attachPropertiesToComponent(Popover, {
  Menu: PopMenu
});