import classNames from 'classnames';
import React, { useEffect, useState, useRef } from 'react';
import { CSSTransition } from 'react-transition-group';
import { useInitialized } from '../../utils/use-initialized';
import { mergeProps } from '../../utils/with-default-props';
import Mask from '../mask';
import { useLockScroll } from '../../utils/use-lock-scroll';
import { renderToContainer } from '../../utils/render-to-container';
var classPrefix = "am-popup";
var defaultProps = {
  position: 'bottom',
  visible: false,
  getContainer: function getContainer() {
    return document.body;
  },
  mask: true
};

var Popup = function Popup(p) {
  var _classNames, _classNames2;

  var props = mergeProps(defaultProps, p); // 动画执行完，才隐藏最外层

  var _useState = useState(!props.visible),
      hidden = _useState[0],
      setHidden = _useState[1];

  useEffect(function () {
    if (props.visible) {
      setHidden(false);
    }
  }, [props.visible]);

  var afterClose = function afterClose() {
    setHidden(true);
    props.afterClose == null ? void 0 : props.afterClose();
  };

  var cls = classNames(classPrefix, props.className, (_classNames = {}, _classNames[classPrefix + "-hidden"] = hidden, _classNames));
  var bodyCls = classNames(classPrefix + "-body", props.bodyClassName, (_classNames2 = {}, _classNames2[classPrefix + "-body-hidden"] = !props.visible, _classNames2), classPrefix + "-body-position-" + props.position);
  var initialized = useInitialized(props.visible || props.forceRender);
  var ref = useRef(null);
  useLockScroll(ref, props.visible);
  var node = /*#__PURE__*/React.createElement("div", {
    className: cls,
    style: props.style
  }, props.mask && /*#__PURE__*/React.createElement(Mask, {
    visible: props.visible,
    onMaskClick: props.onMaskClick,
    className: props.maskClassName,
    style: props.maskStyle,
    disableBodyScroll: false
  }), /*#__PURE__*/React.createElement(CSSTransition, {
    classNames: classPrefix + "-body",
    "in": props.visible,
    timeout: 200,
    onEntered: props.afterShow,
    onExited: afterClose,
    unmountOnExit: props.destroyOnClose
  }, /*#__PURE__*/React.createElement("div", {
    className: bodyCls,
    style: props.bodyStyle,
    ref: ref
  }, initialized && props.children)));
  return renderToContainer(props.getContainer, node);
};

export default Popup;