import { withDefaultProps } from '../../utils/with-default-props';
import { useSpring, animated } from '@react-spring/web';
import { useDrag } from 'react-use-gesture';
import { getScrollParent } from '../../utils/get-scroll-parent';
import React, { useRef, useState } from 'react';
import { supportsPassive } from '../../utils/supports-passive';
import { convertPx } from '../../utils/convert-px';
import { rubberbandIfOutOfBounds } from '../../utils/rubberband';
import { sleep } from '../../utils/sleep';

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var classPrefix = "am-pull-to-refresh";

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

var PullStatus;

function _empty() {}

function _awaitIgnored(value, direct) {
  if (!direct) {
    return value && value.then ? value.then(_empty) : Promise.resolve();
  }
}

function _invoke(body, then) {
  var result = body();

  if (result && result.then) {
    return result.then(then);
  }

  return then(result);
}

function _continue(value, then) {
  return value && value.then ? value.then(then) : then(value);
}

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

(function (PullStatus) {
  PullStatus[PullStatus["idle"] = 0] = "idle";
  PullStatus[PullStatus["thresholdMet"] = 1] = "thresholdMet";
  PullStatus[PullStatus["refreshing"] = 2] = "refreshing";
  PullStatus[PullStatus["complete"] = 3] = "complete";
})(PullStatus || (PullStatus = {}));

export var defaultProps = {
  pullingText: '下拉刷新',
  releaseText: '释放立即刷新',
  refreshingText: '加载中……',
  completeText: '刷新成功',
  completeDelay: 500,
  onRefresh: function onRefresh() {}
};
var PullToRefresh = withDefaultProps(defaultProps)(function (props) {
  var _props$headHeight, _props$threshold;

  var doRefresh = _async(function () {
    var _exit = false;
    api.start({
      height: headHeight
    });
    setStatus(PullStatus.refreshing);
    setStatus(PullStatus.refreshing);
    return _continue(_catch(function () {
      return _await(props.onRefresh(), function () {
        setStatus(PullStatus.complete);
      });
    }, function (e) {
      setStatus(PullStatus.idle);
      throw e;
    }), function (_result) {
      return _exit ? _result : _invoke(function () {
        if (props.completeDelay > 0) {
          return _awaitIgnored(sleep(props.completeDelay));
        }
      }, function () {
        api.start({
          to: _async(function (next) {
            return _await(next({
              height: 0
            }), function () {
              return _await(next({
                height: 0
              }), function () {
                setStatus(PullStatus.idle);
              });
            });
          })
        });
      });
    });
  });

  var headHeight = (_props$headHeight = props.headHeight) != null ? _props$headHeight : convertPx(40);
  var threshold = (_props$threshold = props.threshold) != null ? _props$threshold : convertPx(60);

  var _useState = useState(PullStatus.idle),
      status = _useState[0],
      setStatus = _useState[1];

  var _useSpring = useSpring(function () {
    return {
      from: {
        height: 0
      },
      config: {
        tension: 300,
        friction: 30,
        clamp: true
      }
    };
  }),
      springStyles = _useSpring[0],
      api = _useSpring[1];

  var elementRef = useRef(null);
  var pullingRef = useRef(false);
  useDrag(function (state) {
    if (status === PullStatus.refreshing || status === PullStatus.complete) return;
    var event = state.event;

    if (state.last) {
      pullingRef.current = false;

      if (status === PullStatus.thresholdMet) {
        doRefresh();
      } else {
        api.start({
          height: 0
        });
      }

      return;
    }

    var _state$movement = state.movement,
        x = _state$movement[0],
        y = _state$movement[1];

    if (state.first) {
      var element = elementRef.current;
      if (!element) return;
      var scrollParent = getScrollParent(element);
      if (!scrollParent) return;
      var top = 'scrollTop' in scrollParent ? scrollParent.scrollTop : scrollParent.pageYOffset;

      if (top <= 0 && y > 0) {
        pullingRef.current = true;
      }
    }

    if (!pullingRef.current) return;

    if (typeof event.cancelable !== 'boolean' || event.cancelable) {
      event.preventDefault();
    }

    event.stopPropagation();
    var height = Math.max(rubberbandIfOutOfBounds(y, 0, 0, headHeight * 5, 0.5), 0);
    api.start({
      height: height
    });
    setStatus(height > threshold ? PullStatus.thresholdMet : PullStatus.idle);
  }, {
    useTouch: true,
    axis: 'y',
    domTarget: elementRef,
    eventOptions: supportsPassive ? {
      passive: false
    } : false
  });
  return /*#__PURE__*/React.createElement(animated.div, {
    ref: elementRef,
    className: classPrefix
  }, /*#__PURE__*/React.createElement(animated.div, {
    style: springStyles,
    className: classPrefix + "-head"
  }, /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-head-content",
    style: {
      height: headHeight
    }
  }, status === PullStatus.idle && props.pullingText, status === PullStatus.thresholdMet && props.releaseText, status === PullStatus.refreshing && props.refreshingText, status === PullStatus.complete && props.completeText)), /*#__PURE__*/React.createElement("div", null, props.children));
});
export default PullToRefresh;