import React, { useContext } from 'react';
import classNames from 'classnames';
import { useControllableValue } from 'ahooks';
import { CheckOutlined } from '@ant-design/icons';
import { Group } from './group';
import { RadioGroupContext } from './group-context';
import { attachPropertiesToComponent } from '../../utils/attach-properties-to-component';
var classPrefix = "am-radio";

var Radio = function Radio(props) {
  var _classNames;

  var groupContext = useContext(RadioGroupContext);

  var _useControllableValue = useControllableValue(props, {
    valuePropName: 'checked',
    defaultValuePropName: 'defaultChecked',
    defaultValue: false
  }),
      checked = _useControllableValue[0],
      setChecked = _useControllableValue[1];

  var disabled = props.disabled;
  var value = props.value;

  if (groupContext && value) {
    checked = groupContext.value.includes(value);

    setChecked = function setChecked(checked) {
      if (checked) {
        groupContext.check(value);
      } else {
        groupContext.uncheck(value);
      }

      props.onChange == null ? void 0 : props.onChange(checked);
    };

    disabled = disabled || groupContext.disabled;
  }

  return /*#__PURE__*/React.createElement("label", {
    className: classNames(classPrefix, props.className, (_classNames = {}, _classNames[classPrefix + "-checked"] = checked, _classNames[classPrefix + "-disabled"] = disabled, _classNames)),
    style: props.style
  }, /*#__PURE__*/React.createElement("input", {
    type: 'radio',
    checked: checked,
    onChange: function onChange(e) {
      setChecked(e.target.checked);
    },
    onClick: function onClick(e) {
      e.stopPropagation();
      e.nativeEvent.stopImmediatePropagation();
    },
    disabled: disabled
  }), /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-icon"
  }, /*#__PURE__*/React.createElement(CheckOutlined, {
    className: classPrefix + "-icon-checked"
  })), props.children && /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-content"
  }, props.children));
};

export default attachPropertiesToComponent(Radio, {
  Group: Group
});