var _excluded = ["className", "style", "defaultValue", "value", "onChange", "rows", "autoSize", "showCount"];

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { forwardRef, useEffect, useImperativeHandle, useRef } from 'react';
import classNames from 'classnames';
import { useControllableValue } from 'ahooks';
var classPrefix = 'am-text-area';
var defaultProps = {
  rows: 2,
  showCount: false,
  autoSize: false
}; // TODO: withDefaultProps 和 forwardRef 配合使用的问题

var TextArea = /*#__PURE__*/forwardRef(function (props, ref) {
  var className = props.className,
      style = props.style,
      outerDefaultValue = props.defaultValue,
      outerValue = props.value,
      outerOnChange = props.onChange,
      rows = props.rows,
      autoSize = props.autoSize,
      showCount = props.showCount,
      textAreaProps = _objectWithoutPropertiesLoose(props, _excluded);

  var _useControllableValue = useControllableValue(props, {
    defaultValue: ''
  }),
      value = _useControllableValue[0],
      setValue = _useControllableValue[1];

  var nativeTextAreaRef = useRef(null);
  useImperativeHandle(ref, function () {
    return {
      clear: function clear() {
        setValue('');
      },
      focus: function focus() {
        var _nativeTextAreaRef$cu;

        (_nativeTextAreaRef$cu = nativeTextAreaRef.current) == null ? void 0 : _nativeTextAreaRef$cu.focus();
      },
      blur: function blur() {
        var _nativeTextAreaRef$cu2;

        (_nativeTextAreaRef$cu2 = nativeTextAreaRef.current) == null ? void 0 : _nativeTextAreaRef$cu2.blur();
      }
    };
  });
  useEffect(function () {
    if (!autoSize) return;
    var textArea = nativeTextAreaRef.current;
    if (!textArea) return;
    textArea.style.height = 'auto';
    var height = textArea.scrollHeight;

    if (typeof autoSize === 'object') {
      var computedStyle = window.getComputedStyle(textArea);
      var lineHeight = parseFloat(computedStyle.lineHeight);

      if (autoSize.minRows) {
        height = Math.max(height, autoSize.minRows * lineHeight);
      }

      if (autoSize.maxRows) {
        height = Math.min(height, autoSize.maxRows * lineHeight);
      }
    }

    textArea.style.height = height + "px";
  }, [value, autoSize]);
  return /*#__PURE__*/React.createElement("div", {
    className: classNames(classPrefix + "-wrapper", className),
    style: style
  }, /*#__PURE__*/React.createElement("textarea", Object.assign({
    ref: nativeTextAreaRef
  }, textAreaProps, {
    className: classPrefix,
    rows: rows,
    value: value,
    onChange: function onChange(e) {
      setValue(e.target.value);
    },
    onFocus: function onFocus(e) {
      props.onFocus == null ? void 0 : props.onFocus(e);
    },
    onBlur: function onBlur(e) {
      props.onBlur == null ? void 0 : props.onBlur(e);
    }
  })), showCount && /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-count"
  }, props.maxLength === undefined ? value.length : value.length + '/' + props.maxLength));
});
TextArea.defaultProps = defaultProps;
export default TextArea;