function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React, { useEffect, useMemo, useState, useCallback } from 'react';
import ReactDOM from 'react-dom';
import classNames from 'classnames';
import { CheckOutlined, CloseOutlined } from '@ant-design/icons';
import { useUpdateEffect } from 'ahooks';
import { noop } from '../../utils/noop';
import Loading from '../loading';
import Mask from '../mask';
import { resolveContainer } from '../../utils/get-container';
var classPrefix = "am-toast";
var toastArray = [];

var InternalToast = function InternalToast(props) {
  var _props$maskClickable = props.maskClickable,
      maskClickable = _props$maskClickable === void 0 ? true : _props$maskClickable,
      content = props.content,
      _props$icon = props.icon,
      icon = _props$icon === void 0 ? null : _props$icon,
      position = props.position;
  var top = useMemo(function () {
    switch (position) {
      case 'top':
        return '20%';

      case 'bottom':
        return '80%';

      default:
        return '50%';
    }
  }, [position]);
  return /*#__PURE__*/React.createElement(Mask, {
    visible: props.visible,
    destroyOnClose: true,
    opacity: 0,
    disableBodyScroll: !maskClickable,
    getContainer: props.getContainer,
    afterClose: props.afterClose,
    style: _extends({
      pointerEvents: maskClickable ? 'none' : 'all'
    }, props.maskStyle),
    className: classNames(classPrefix + "-mask", props.maskClassName)
  }, /*#__PURE__*/React.createElement("div", {
    style: {
      top: top
    },
    className: classNames(classPrefix + "-wrap", icon ? classPrefix + "-wrap-icon" : classPrefix + "-wrap-text")
  }, Boolean(icon) && /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-icon"
  }, icon), content));
}; // 可返回用于销毁此弹窗的方法


function show(props) {
  var updateConfig = function updateConfig() {};

  var timer = 0;
  var _props$afterClose = props.afterClose,
      afterClose = _props$afterClose === void 0 ? noop : _props$afterClose,
      _props$getContainer = props.getContainer,
      getContainer = _props$getContainer === void 0 ? function () {
    return document.body;
  } : _props$getContainer;
  var container = document.createElement('div');
  var bodyContainer = resolveContainer(getContainer);
  bodyContainer.appendChild(container);

  var TempToast = function TempToast() {
    var _useState = useState(true),
        visible = _useState[0],
        setVisible = _useState[1];

    var _useState2 = useState(_extends({
      duration: 2000
    }, props)),
        state = _useState2[0],
        setState = _useState2[1]; // clearDOM after animation


    var _afterClose = function _afterClose() {
      afterClose();
      var unmountResult = ReactDOM.unmountComponentAtNode(container);

      if (unmountResult && container.parentNode) {
        container.parentNode.removeChild(container);
      }
    }; // close with animation


    var destroy = useCallback(function () {
      setVisible(false);
    }, []);
    useEffect(function () {
      _clear();

      toastArray.push(_afterClose);
    }, []);
    updateConfig = useCallback(function (nextState) {
      return setState(function (prev) {
        return typeof nextState === 'function' ? _extends({}, prev, nextState(prev)) : _extends({}, prev, nextState);
      });
    }, [setState]);
    useEffect(function () {
      if (state.duration !== 0 && 'duration' in state) {
        timer = window.setTimeout(destroy, state.duration);
      }

      return function () {
        if (timer !== 0) {
          window.clearTimeout(timer);
        }
      };
    }, []); // 当修改 duration 时，重新计时

    useUpdateEffect(function () {
      if ('duration' in state) {
        window.clearTimeout(timer);
        timer = window.setTimeout(destroy, state.duration);
      }
    }, [state.duration]);
    return /*#__PURE__*/React.createElement(InternalToast, Object.assign({}, state, {
      getContainer: function getContainer() {
        return container;
      },
      visible: visible,
      afterClose: _afterClose
    }));
  };

  ReactDOM.render( /*#__PURE__*/React.createElement(TempToast, null), container);
  return updateConfig;
}

function success(props) {
  return show(_extends({
    icon: /*#__PURE__*/React.createElement(CheckOutlined, null)
  }, props));
}

function fail(props) {
  return show(_extends({
    icon: /*#__PURE__*/React.createElement(CloseOutlined, null)
  }, props));
}

function loading(props) {
  return show(_extends({
    icon: /*#__PURE__*/React.createElement(Loading, {
      color: 'white'
    })
  }, props));
} // 同步的销毁


function _clear() {
  var fn = toastArray.pop();

  while (fn) {
    fn();
    fn = toastArray.pop();
  }
} // 针对 toast 还没弹出来就立刻销毁的情况，将销毁放到下一个 event loop 中，避免销毁失败。


function clear() {
  setTimeout(_clear);
}

var Toast = {
  show: show,
  success: success,
  fail: fail,
  loading: loading,
  clear: clear
};
export default Toast;